"""
Module Info Message
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy

class InfoMessage(domipy.Message):
    """
    Generic info message
    """

    def __init__(self, moduleType=None, data=None):

        domipy.Message.__init__(self)

        self._message = ''
        self.moduleType = 'INFO'
        self._message = data
    
    def populate(self, serialNumber, dataType, dataString):
        pass

    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        json_dict['info_message'] = self._message
        return json.dumps(json_dict)

domipy.register_command("INF", InfoMessage)
domipy.register_command("!! ", InfoMessage)
domipy.register_command("APP", InfoMessage)


"""
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy


class Ping(domipy.Command):
    """
        send: &PING message
    """
    def __init__(self):
        domipy.Command.__init__(self)

    def command(self):
        return "PING"



"""
DISM4 & DISM8 module status
Communication module.
This module permits the direct connection of 1 to 4 push buttons (DISM04) 
or from 1 to 8 push buttons (DISM08) or any other inputs, detectors, etc. 
Each DISM0x has a unique number which enables it to be 

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
from domipy.messages.di_status import GenericDIStatusMessage

IS4_COMMAND_CODE = "IS4"
IS8_COMMAND_CODE = "IS8"

class DISM4StatusMessage(GenericDIStatusMessage):
    """
    DISM4 module status
    """

    def __init__(self, address=None):
        GenericDIStatusMessage.__init__(self, IS4_COMMAND_CODE, 4)

class DISM8StatusMessage(GenericDIStatusMessage):
    """
    DISM8 module status
    """
    def __init__(self, address=None):
        GenericDIStatusMessage.__init__(self, IS8_COMMAND_CODE, 8)

domipy.register_command(IS4_COMMAND_CODE, DISM4StatusMessage)
domipy.register_command(IS8_COMMAND_CODE, DISM8StatusMessage)



"""
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy

MODULE_DIRECTORY = {
    'IS4': { 'type': 'DISM4', 'name': 'Control of 4 inputs (buttons)', 'io': 'di'},
    'IS8': { 'type': 'DISM8', 'name': 'Control of 8 inputs (buttons)', 'io': 'di'},
    'BU1': { 'type': 'DPBU01', 'name': '1 button module', 'io': 'di'},
    'BU2': { 'type': 'DPBU02', 'name': '2 button module', 'io': 'di'},
    'BU4': { 'type': 'DPBU04', 'name': '4 button module', 'io': 'di'},
    'BU6': { 'type': 'DPBU06', 'name': '6 button module', 'io': 'di'},
    'BIR': { 'type': 'DBIR01', 'name': '8 relay output module', 'io': 'do'},
    'TRP': { 'type': 'DTRP01', 'name': 'Output card for the control of up to 4 trip switches', 'io': 'do'},
    'TPV': { 'type': 'DTRP02', 'name': 'Output card for the control of 2 x 2 inverted trip switches', 'io': 'do'},    
    'DIM': { 'type': 'DDIM01', 'name': 'Control module for up to 8 dimmers', 'io': 'ao'},
    'TE1': { 'type': 'DTEM01', 'name': 'Temperature sensor module. Allows the connection of the temperature sensor', 'io': 't'},
    'TE2': { 'type': 'DTEM02', 'name': 'Temperature sensor module. Allows the connection of the temperature sensor', 'io': 't'},
    'DIR': { 'type': 'DDIR01', 'name': 'Infrared Sensor', 'io': 'ai'},
    'TSB': { 'type': 'DTSC0x', 'name': 'TFT back-lit color touchscreen', 'io': 't'},
    'DET': { 'type': 'DMOV01', 'name': 'Movement sensor', 'io': 'di'},
    'LCD': { 'type': 'DLCD01', 'name': 'LCD', 'io': 'unk'},
    'TRV': { 'type': 'DTRV01', 'name': '4 outputs control module. For the control of shutters, valves, motors, etc', 'io': 'do'},
    'LED': { 'type': 'DLED01', 'name': '4 outputs led control module', 'io': 'do'},
    'V24': { 'type': 'DTRVBT01', 'name': 'Single output card controlling motors, valves, shutters or Velux with low tension between 12 to 24Vdc', 'io': 'do'},
    'AMP': { 'type': 'DAMPLI01', 'name': ' 4 zones stereo audio amplifi er ', 'io': 'unk'},
    'VAR': { 'type': 'VAR', 'name': 'Variable', 'io': 'var'},
    'SYS': { 'type': 'SYS', 'name': 'System variable', 'io': 'var'},
    'D10': { 'type': 'DOUT10V01', 'name': '0 - 10Vdc output module', 'io': 'ao'},
    'LC3': { 'type': 'DLCD01', 'name': 'LCD panel', 'io': 'di'},
    'FAN': { 'type': 'DFAN01', 'name': 'Fan coil controller', 'io': 'do'},
    'DMR': { 'type': 'DMR01', 'name': 'Output card with 5 x 250 V/3 A monopolar relays.', 'io': 'do'},
    'I10': { 'type': 'DIN10V01', 'name': '0 - 10Vdc input module', 'io': 'ai'},
    'SFE': { 'type': 'SFE', 'name': 'Atmosphere', 'io': 'unk'},
    'ZON': { 'type': 'ZON', 'name': 'Zone', 'io': 'unk'},
    'MEM': { 'type': 'MEM', 'name': 'Memo', 'io': 'unk'},
    'TPR': { 'type': 'TPR', 'name': 'Heating Schedule', 'io': 'unk'}
}

def get_point_id(module_code, serial_number, channel=-1):
    # assert isinstance(module_code, str)
    assert isinstance(serial_number, str)
    # hw_id = module_code.strip()
    se_no = serial_number.strip()

    pid = se_no

    if not isinstance(channel, int):
        channel = -1
        
    if channel > -1 and channel < 8:
        channel += 1  # we use 0 based index internally confert to 1 based for domintell
        pid = "{}-{}".format(se_no, channel)

    return pid



"""
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy


class SetTemperatureMessage(domipy.Command):
    """
    Set temperature
    """
    def __init__(self, moduleType, serialNumber, value):
        domipy.Command.__init__(self, moduleType, serialNumber.strip(), -1, "%T", value)

class SetTemperatureModeMessage(domipy.Command):
    """
    Set temperature mode
    1 - Absense
    2 - Automatic
    5 - COmfort
    6 - Frost
    """
    def __init__(self, moduleType, serialNumber, value):
        domipy.Command.__init__(self, moduleType, serialNumber.strip(), -1,  "%M", value)

class SetTemperatureAutomaticMessage(domipy.Command):
    """
    Set AUTOMATIC mode
    """
    def __init__(self, moduleType, serialNumber):
        domipy.Command.__init__(self, moduleType, serialNumber.strip(), -1, "%M", 2)

class SetTemperatureAbsenceMessage(domipy.Command):
    """
    Set ABSEBSE mode
    """
    def __init__(self, moduleType, serialNumber):
        domipy.Command.__init__(self, moduleType, serialNumber.strip(), -1, "%M", 1)

class SetTemperatureComfortMessage(domipy.Command):
    """
    Set COMFORT mode
    """
    def __init__(self, moduleType, serialNumber):
        domipy.Command.__init__(self, moduleType, serialNumber.strip(), -1, "%M", 5)

class SetTemperatureFrostMessage(domipy.Command):
    """
    Set FROST mode
    """
    def __init__(self, moduleType, serialNumber):
        domipy.Command.__init__(self, moduleType, serialNumber.strip(), -1, "%M", 6)

class SetTemperatureSetPointMessage(domipy.Command):
    """
    Set temperature
    """
    def __init__(self, moduleType, serialNumber, value):
        domipy.Command.__init__(self, moduleType, serialNumber.strip(), -1, "%T", value)



"""
TRV module status
Output card to control 8 motors

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
from domipy.messages import GenericDOStatusMessage

TRV_COMMAND_CODE = "TRV"
V24_COMMAND_CODE = "V24"

class DTRVStatusMessage(GenericDOStatusMessage):
    """
    DBIR module status
    """

    def __init__(self, address=None):
        GenericDOStatusMessage.__init__(self, 8)
        self.moduleType = TRV_COMMAND_CODE

class DTRVBTStatusMessage(GenericDOStatusMessage):
    """
    DBIR module status
    """

    def __init__(self, address=None):
        GenericDOStatusMessage.__init__(self, 1)
        self.moduleType = V24_COMMAND_CODE


domipy.register_command(TRV_COMMAND_CODE, DTRVStatusMessage)
domipy.register_command(V24_COMMAND_CODE, DTRVBTStatusMessage)



"""
:author: Thomas Delaet <thomas@delaet.org>
"""
# pylint: disable-msg=C0301
from domipy.modules.ddim01 import DDIM01Module
from domipy.modules.dism0x import DISM04Module
from domipy.modules.dism0x import DISM08Module
from domipy.modules.do import DBIR01Module
from domipy.modules.do import DTRP01Module
from domipy.modules.do import DTRP02Module
from domipy.modules.do import DTRV01Module
from domipy.modules.do import DTRVBT01Module
from domipy.modules.do import DFAN01Module
from domipy.modules.do import DMR01Module
from domipy.modules.do import DLED01Module
from domipy.modules.dpbu0x import DPBU01Module
from domipy.modules.dpbu0x import DPBU02Module
from domipy.modules.dpbu0x import DPBU04Module
from domipy.modules.dpbu0x import DPBU06Module
from domipy.modules.dtemp import DTEM01Module
from domipy.modules.dtemp import DTEM02Module
 


"""
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import hashlib
import domipy

ENCODING = "utf-8"

class LoginRequest(domipy.Command): 
    """
        send: LOGIN<password_hash>
    """
    def __init__(self, username, password, salt, nonce):
        domipy.Command.__init__(self, "_LOGIN_", "_LOGIN_")
        self._password = password
        self._username = username
        self._salt = salt
        self._nonce = nonce

    def command(self):
        salted_pass = self._password+self._salt
        m = hashlib.sha512()
        m.update(bytes(salted_pass, ENCODING))
        hash = m.hexdigest()
        m2 = hashlib.sha512()
        m2.update(bytes(hash+self._nonce,ENCODING))

        return "LOGINPSW@{}:{}".format(self._username, m2.hexdigest())
        
    def is_binary(self):
        return True





"""
VAR & SYS

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
from domipy.messages import GenericDIStatusMessage

VAR_COMMAND_CODE = "VAR"
SYS_COMMAND_CODE = "SYS"


class GenericVARStatusMessage(GenericDIStatusMessage):
    """
    Generic VAR module status
    """
    def __init__(self, command_code=VAR_COMMAND_CODE, address=None):
        GenericDIStatusMessage.__init__(self, command_code, 1)
        self.moduleType = command_code
        self.serialNumber = None
        self.dataType = None
        self.var = 0
        self.inputs = {}

    def populate(self, serialNumber, dataType, dataString):
        # Since dimintell can have the same numbers for VAR and SYS
        # we can not identify variable by serialNumber,
        # so we prepend moduleType/command_code in the front
        if dataType == "O":
            self.populateA(serialNumber, dataType, dataString)
        elif dataType == "D":
            self.populateD(self.moduleType + serialNumber, dataType, dataString)

    def populateD(self, serialNumber, dataType, dataString):
        assert isinstance(dataString, str)
        self.serialNumber = serialNumber
        self.dataType = dataType
        if dataString[0:2].strip() == "00":
            self.var = 0
        else:
            self.var = 1
        self.inputs[0] = self.var

    def populateA(self, serialNumber, dataType, dataString):
        self.serialNumber = serialNumber
        self.dataType = dataType
        self.var = int(dataString[0:2].strip(), 16)
        self.inputs[0] = self.var

    def to_json(self):
        """
        :return: str
        """
        json_dict = self.to_json_basic()
        json_dict['var'] = self.var
        return json.dumps(json_dict)


class VARStatusMessage(GenericVARStatusMessage):
    """
    VAR  status
    """
    def __init__(self, address=None):
        GenericVARStatusMessage.__init__(self, VAR_COMMAND_CODE)


class SYSStatusMessage(GenericVARStatusMessage):
    """
    SYS  status
    """
    def __init__(self, address=None):
        GenericVARStatusMessage.__init__(self, SYS_COMMAND_CODE)



domipy.register_command(VAR_COMMAND_CODE, VARStatusMessage)
domipy.register_command(SYS_COMMAND_CODE, SYSStatusMessage)



"""
Dimmer control module DDIM01
Controlls up to 8 dimmers DD500, DD750, DD1000 and DD10V

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy
import domipy.messages

class GenericDPBU0xModule(domipy.Module):
    """
    Abstract DPBU control module     
    """
    COMMAND_CODE = 'UNK'

    def __init__(self, serial_number, controller):
        domipy.Module.__init__(self, serial_number, controller)
        self._leds = {}
        for i in range(0, self.number_of_channels()):
            self._leds[i] = 0


    def is_on(self, channel):
        if channel < self.number_of_channels():
            if self._values[channel] > 0:
                return True
        return False

    def is_led_on(self, channel):
        if channel < self.number_of_channels():
            if self._leds[channel] > 0:
                return True
        return False
        
    def sim_long_push(self, channel):
        """
        Simulate long push
        """
        # TODO
        pass

    def sim_long_push_end(self, channel):
        """ 
        Simulate end of long push
        """
        # TODO
        pass

    def sim_short_push(self, channel):
        """
        Simulate short push
        """
        # TODO
        pass

    def sim_short_push_end(self, channel):
        """ 
        Simulate end of short push
        """
        # TODO
        pass

    def number_of_channels(self):
        raise NotImplementedError

    def _on_message(self, message):
        if isinstance(message, domipy.DDIMStatusMessage):
            
            if message.is_input():
                self._values = message.get_inputs()

            if message.is_output():
                self._leds = message.get_outputs()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self._values[ch], self._leds[ch])

class DPBU01Module(GenericDPBU0xModule):

    COMMAND_CODE = 'BU1'

    def number_of_channels(self):
        return 1

    def _on_message(self, message):
        if isinstance(message, domipy.DPBU01StatusMessage):
            
            if message.is_input():
                self._values = message.get_inputs()

            if message.is_output():
                self._leds = message.get_outputs()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self._values[ch], self._leds[ch])

class DPBU02Module(GenericDPBU0xModule):

    COMMAND_CODE = 'BU2'

    def number_of_channels(self):
        return 2

    def _on_message(self, message):
        if isinstance(message, domipy.DPBU02StatusMessage):
            
            if message.is_input():
                self._values = message.get_inputs()

            if message.is_output():
                self._leds = message.get_outputs()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self._values[ch], self._leds[ch])

class DPBU04Module(GenericDPBU0xModule):

    COMMAND_CODE = 'BU4'

    def number_of_channels(self):
        return 4

    def _on_message(self, message):
        if isinstance(message, domipy.DPBU04StatusMessage):
            
            if message.is_input():
                self._values = message.get_inputs()

            if message.is_output():
                self._leds = message.get_outputs()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self._values[ch], self._leds[ch])

class DPBU06Module(GenericDPBU0xModule):

    COMMAND_CODE = 'BU6'

    def number_of_channels(self):
        return 6

    def _on_message(self, message):
        if isinstance(message, domipy.DPBU06StatusMessage):
            
            if message.is_input():
                self._values = message.get_inputs()

            if message.is_output():
                self._leds = message.get_outputs()

            for ch in range(0, self.number_of_channels()):
                if ch in self._callbacks:
                    for callback in self._callbacks[ch]:
                        callback(self._values[ch], self._leds[ch])


domipy.register_module_class(DPBU01Module)
domipy.register_module_class(DPBU02Module)
domipy.register_module_class(DPBU04Module)
domipy.register_module_class(DPBU06Module)



from setuptools import setup

with open('README.md') as file:
    long_description = file.read()

setup(name='domipy',
      version='0.0.14',
      url='https://github.com/soliveira/python-domintell',
      license='MIT',
      author='Sérgio Oliveira',
      install_requires=["pyserial","websocket-client"],
      author_email='so@soliveira.pt',
      description="A port of the python Library for the Domintell protocol using Secure Web Sockets and new login mecanism. Original work from Zilvinas Binisevicius (zilvinas@binis.me)",
      long_description=long_description,
      long_description_content_type="text/markdown",
      packages=['domipy', 'domipy.utils', 'domipy.connections', 'domipy.messages', 'domipy.modules'],
      platforms='any'
     )



"""
BIR module status
Output card to control 8 250V/8A two-pole relays.

:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import json
import domipy
from domipy.messages import GenericAIStatusMessage

DIN10V_COMMAND_CODE = "I10"

class DIN10VStatusMessage(GenericAIStatusMessage):
    """
    I10 module status
    """
    def __init__(self, address=None):
        GenericAIStatusMessage.__init__(self, 10)
        self.moduleType = DIN10V_COMMAND_CODE

domipy.register_command(DIN10V_COMMAND_CODE, DIN10VStatusMessage)



"""
:author: Zilvinas Binisevicius <zilvinas@binis.me>
"""
import domipy
from domipy.messages import SetAnalogOutputMessage

class SetDimmer(SetAnalogOutputMessage):

    def __init__(self, serialNumber, value):
        SetAnalogOutputMessage.__init__(self, "DIM", serialNumber, "%D", value)

class StartDimmer(domipy.Command):
    """
        Start dimmer action
    """
    def __init__(self, serialNumber):
        domipy.Command.__init__(self, "DIM", serialNumber, "%DB")

class StopDimmer(domipy.Command):
    """
        Stop dimmer action
    """
    def __init__(self, serialNumber):
        domipy.Command.__init__(self, "DIM", serialNumber, "%DE")

class IncrementDimmer(domipy.Command):
    """
        Increment dimmer
    """
    def __init__(self, serialNumber, value):
        domipy.Command.__init__(self, "DIM", serialNumber, "%I%D", value)

class DecrementDimmer(domipy.Command):
    """
        Decrement dimmer
    """
    def __init__(self, serialNumber, value):
        domipy.Command.__init__(self, "DIM", serialNumber, "%O%D", value)
